SystemOrganization addCategory: #'Neo-Console-Core'!

Object subclass: #NeoConsoleTelnetServer
	instanceVariableNames: 'process serverSocket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-Console-Core'!

!NeoConsoleTelnetServer commentStamp: 'SvenVanCaekenberghe 2/18/2015 11:31' prior: 0!
NeoConsoleTelnetServer offers a command line interface to a Pharo image.

Inspect:

  NeoConsoleTelnetServer new start.

In a terminal do:

  $ telnet localhost 4999

End with quit or EOF.!

!NeoConsoleTelnetServer methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 2/11/2015 23:31'!
executeRequestResponseLoopOn: stream
	| input |
	self writeLine: ('Neo Console ', SystemVersion current asString) to: stream.
	[
		(input := (self readInputFrom: stream))
		ifNotNil: [ self writeOutput: ('You said: ', String cr, input) to: stream ].
		input notNil	 ] whileTrue! !

!NeoConsoleTelnetServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/11/2015 16:09'!
initializeServerSocket
	serverSocket := Socket newTCP.
	serverSocket setOption: 'TCP_NODELAY' value: 1.
	serverSocket 
		listenOn: self port 
		backlogSize: 10 
		interface: NetNameResolver loopBackAddress.
	serverSocket isValid
		ifFalse: [ self error: 'Cannot create socket on port ' , self port printString ]! !

!NeoConsoleTelnetServer methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/11/2015 15:49'!
isRunning
	"Return true when I am running"
	
	^ process notNil and: [ serverSocket notNil ]! !

!NeoConsoleTelnetServer methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 2/11/2015 16:00'!
listenLoop
	"We create a listening Socket, then wait for a connection.
	After each connection we also check that the listening Socket is still valid 
	- if not we just make a recursive call to this method to start over."

	self initializeServerSocket.
	[ [ 
		serverSocket isValid 
			ifFalse: [
				"will trigger #ifCurtailed: block and destroy socket"
				^ self listenLoop ].
		self serveConnectionsOn: serverSocket ] repeat ]

		ifCurtailed: [ self releaseServerSocket ]! !

!NeoConsoleTelnetServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/11/2015 15:48'!
port
	^ 4999! !

!NeoConsoleTelnetServer methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 2/11/2015 23:31'!
readInputFrom: stream
	| input lineReader line |
	stream nextPutAll: '> '; flush.
	input := String new writeStream.
	lineReader := ZnLineReader on: stream.
	[ 
		[ line := lineReader nextLine ] 
			on: ConnectionClosed, ConnectionTimedOut do: [ ^ nil ].
		(line trimBoth beginsWith: 'quit') 
			ifTrue: [ 
				self writeLine: 'Bye!!' to: stream. 
				^ nil ].
		input nextPutAll: line; cr.
		line isEmpty ] whileFalse.
	^ input contents! !

!NeoConsoleTelnetServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/11/2015 16:04'!
releaseServerSocket
	(Delay forMilliseconds: 10) wait.
	serverSocket destroy.
	serverSocket := nil! !

!NeoConsoleTelnetServer methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 2/11/2015 16:12'!
serveConnectionsOn: listeningSocket
	"We wait up to acceptWaitTimeout seconds for an incoming connection.
	If we get one we wrap it in a SocketStream and #executeRequestResponseLoopOn: on it"
		 
	| stream socket |
	socket := listeningSocket waitForAcceptFor: 300.
	socket ifNil: [ ^ self ].
	stream := self socketStreamOn: socket.
	[ [ [ self executeRequestResponseLoopOn: stream ]
		ensure: [ [ stream close ] on: Exception do: [ ] ] ]
			ifCurtailed: [ socket destroy ] ]
				forkAt: Processor lowIOPriority
				named: self workerProcessName ! !

!NeoConsoleTelnetServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/11/2015 15:51'!
serverProcessName
	^ self class name asString! !

!NeoConsoleTelnetServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/11/2015 16:10'!
socketStreamOn: socket
	^ (ZdcSocketStream on: socket)
		binary;
		shouldSignal: true;
		autoFlush: false;
		timeout: self timeout;
		yourself! !

!NeoConsoleTelnetServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 2/11/2015 15:51'!
start
	self stop.
	process := [ [ self listenLoop ] repeat ] 
					forkAt: Processor highIOPriority 
					named: self serverProcessName! !

!NeoConsoleTelnetServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 2/11/2015 15:50'!
stop
	self isRunning ifFalse: [ ^ self ].
	process terminate. 
	process := nil.
! !

!NeoConsoleTelnetServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/11/2015 16:10'!
timeout
	^ 30! !

!NeoConsoleTelnetServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/11/2015 16:07'!
workerProcessName
	^ self serverProcessName, ' Connection Handler'! !

!NeoConsoleTelnetServer methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 2/11/2015 23:31'!
writeLine: string to: stream
	stream nextPutAll: string.
	stream nextPutAll: String crlf.
	stream flush! !

!NeoConsoleTelnetServer methodsFor: 'protocol' stamp: 'SvenVanCaekenberghe 2/11/2015 23:21'!
writeOutput: string to: stream
	string lines
		do: [ :each | stream nextPutAll: each ]
		separatedBy: [ stream nextPutAll: String crlf ].
	stream flush! !
